@@ -48,10 +48,6 @@
 
 #include <glib/gstdio.h>
 
-#ifdef USE_GCONF
-#include <gconf/gconf-client.h>
-#endif
-
 #include <libgnomeui/gnome-icon-lookup.h>
 #include <libgnomeui/gnome-authentication-manager-private.h>
 
@@ -60,6 +56,50 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
+#include "sucky-desktop-item.h"
+
+#undef PROFILE_FILE_CHOOSER
+#ifdef PROFILE_FILE_CHOOSER
+#define PROFILE_INDENT 4
+static int profile_indent;
+
+static void
+profile_add_indent (int indent)
+{
+  profile_indent += indent;
+  if (profile_indent < 0)
+    g_error ("You screwed up your indentation");
+}
+
+static void
+profile_log (const char *func, int indent, const char *msg1, const char *msg2)
+{
+  char *str;
+
+  if (indent < 0)
+    profile_add_indent (indent);
+
+  if (profile_indent == 0)
+    str = g_strdup_printf ("MARK: %s %s %s", func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+  else
+    str = g_strdup_printf ("MARK: %*c %s %s %s", profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+
+  access (str, F_OK);
+  g_free (str);
+
+  if (indent > 0)
+    profile_add_indent (indent);
+}
+
+#define profile_start(x, y) profile_log (G_STRFUNC, PROFILE_INDENT, x, y)
+#define profile_end(x, y) profile_log (G_STRFUNC, -PROFILE_INDENT, x, y)
+#define profile_msg(x, y) profile_log (NULL, 0, x, y)
+#else
+#define profile_start(x, y)
+#define profile_end(x, y)
+#define profile_msg(x, y)
+#endif
 
 #define BOOKMARKS_FILENAME ".gtk-bookmarks"
 #define BOOKMARKS_TMP_FILENAME ".gtk-bookmarks-XXXXXX"
@@ -97,12 +137,6 @@ struct _GtkFileSystemGnomeVFS
   gulong drive_connected_id;
   gulong drive_disconnected_id;
 
-#ifdef USE_GCONF
-  GConfClient *client;
-  guint client_notify_id;
-  GSList *bookmarks;
-#endif
-
   char *desktop_uri;
   char *home_uri;
 
@@ -169,12 +203,6 @@ struct _FolderChild
  * for them.
  */
 
-#ifdef USE_GCONF
-/* GConf paths for the bookmarks; keep these two in sync */
-#define BOOKMARKS_KEY  "/desktop/gnome/interface/bookmark_folders"
-#define BOOKMARKS_PATH "/desktop/gnome/interface"
-#endif
-
 static void gtk_file_system_gnome_vfs_class_init (GtkFileSystemGnomeVFSClass *class);
 static void gtk_file_system_gnome_vfs_iface_init (GtkFileSystemIface         *iface);
 static void gtk_file_system_gnome_vfs_init       (GtkFileSystemGnomeVFS      *impl);
@@ -271,7 +299,8 @@ static gboolean     gtk_file_folder_gnome_vfs_is_finished_loading (GtkFileFolder
 
 static GtkFileInfo *           info_from_vfs_info (const gchar      *uri,
 						   GnomeVFSFileInfo *vfs_info,
-						   GtkFileInfoType   types);
+						   GtkFileInfoType   types,
+						   GError          **error);
 static GnomeVFSFileInfoOptions get_options        (GtkFileInfoType   types);
 
 static void volume_mount_unmount_cb (GnomeVFSVolumeMonitor *monitor,
@@ -292,17 +321,6 @@ static void monitor_callback        (GnomeVFSMonitorHandle    *handle,
 				     GnomeVFSMonitorEventType  event_type,
 				     gpointer                  user_data);
 
-#ifdef USE_GCONF
-static void set_bookmarks_from_value (GtkFileSystemGnomeVFS *system_vfs,
-				      GConfValue            *value,
-				      gboolean               emit_changed);
-
-static void client_notify_cb (GConfClient *client,
-			      guint        cnxn_id,
-			      GConfEntry  *entry,
-			      gpointer     data);
-#endif
-
 static gchar *make_child_uri (const gchar *base_uri,
 			      const gchar *child_name,
 			      GError     **error);
@@ -401,10 +419,8 @@ static void
 gtk_file_system_gnome_vfs_init (GtkFileSystemGnomeVFS *system_vfs)
 {
   char *name;
-#ifdef USE_GCONF
-  GConfValue *value;
-#endif
 
+  profile_start ("start", NULL);
 
   bindtextdomain (GETTEXT_PACKAGE, GNOMEUILOCALEDIR);
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET 
@@ -420,26 +436,6 @@ gtk_file_system_gnome_vfs_init (GtkFileSystemGnomeVFS *system_vfs)
   system_vfs->locale_encoded_filenames = (getenv ("G_BROKEN_FILENAMES") != NULL);
   system_vfs->folders = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
-#ifdef USE_GCONF
-  system_vfs->client = gconf_client_get_default ();
-  system_vfs->bookmarks = NULL;
-
-  value = gconf_client_get (system_vfs->client, BOOKMARKS_KEY, NULL);
-  set_bookmarks_from_value (system_vfs, value, FALSE);
-  if (value)
-    gconf_value_free (value);
-
-  /* FIXME: use GError? */
-  gconf_client_add_dir (system_vfs->client, BOOKMARKS_PATH, GCONF_CLIENT_PRELOAD_NONE, NULL);
-
-  system_vfs->client_notify_id = gconf_client_notify_add (system_vfs->client,
-							  BOOKMARKS_KEY,
-							  client_notify_cb,
-							  system_vfs,
-							  NULL,
-							  NULL);
-#endif
-
   system_vfs->volume_monitor = gnome_vfs_get_volume_monitor ();
   system_vfs->volume_mounted_id =
     g_signal_connect_object (system_vfs->volume_monitor, "volume-mounted",
@@ -465,6 +461,8 @@ gtk_file_system_gnome_vfs_init (GtkFileSystemGnomeVFS *system_vfs)
     system_vfs->have_net = TRUE;
   else
     system_vfs->have_net = FALSE;
+
+  profile_end ("end", NULL);
 }
 
 static void
@@ -485,12 +483,6 @@ gtk_file_system_gnome_vfs_finalize (GObject *object)
   g_hash_table_foreach (system_vfs->folders, unref_folder, NULL);
   g_hash_table_destroy (system_vfs->folders);
 
-#ifdef USE_GCONF
-  gconf_client_notify_remove (system_vfs->client, system_vfs->client_notify_id);
-  g_object_unref (system_vfs->client);
-  gtk_file_paths_free (system_vfs->bookmarks);
-#endif
-
   g_signal_handler_disconnect (system_vfs->volume_monitor, system_vfs->volume_mounted_id);
   g_signal_handler_disconnect (system_vfs->volume_monitor, system_vfs->volume_unmounted_id);
   g_signal_handler_disconnect (system_vfs->volume_monitor, system_vfs->drive_connected_id);
@@ -515,6 +507,8 @@ gtk_file_system_gnome_vfs_list_volumes (GtkFileSystem *file_system)
   GList *list;
   GList *l;
 
+  profile_start ("start", NULL);
+
   result = NULL;
 
   /* User-visible drives */
@@ -563,6 +557,8 @@ gtk_file_system_gnome_vfs_list_volumes (GtkFileSystem *file_system)
   if (volume)
     result = g_slist_prepend (result, volume);
 
+  profile_end ("end", NULL);
+
   return result;
 }
 
@@ -574,9 +570,14 @@ gtk_file_system_gnome_vfs_get_volume_for_path (GtkFileSystem     *file_system,
   GnomeVFSURI *uri;
   GnomeVFSVolume *volume;
 
+  profile_start ("start", (char *) path);
+
   uri = gnome_vfs_uri_new (gtk_file_path_get_string (path));
   if (!uri)
-    return NULL;
+    {
+      profile_end ("end", (char *) path);
+      return NULL;
+    }
 
   volume = NULL;
 
@@ -603,6 +604,8 @@ gtk_file_system_gnome_vfs_get_volume_for_path (GtkFileSystem     *file_system,
   if (uri)
     gnome_vfs_uri_unref (uri);
 
+  profile_end ("end", (char *) path);
+
   return (GtkFileSystemVolume *) volume;
 }
 
@@ -624,6 +627,8 @@ load_dir (GtkFileFolderGnomeVFS *folder_vfs)
   int num_items;
   GnomeVFSFileInfoOptions vfs_options;
 
+  profile_start ("start", folder_vfs->uri);
+
   if (folder_vfs->async_handle)
     {
       gnome_vfs_async_cancel (folder_vfs->async_handle);
@@ -649,6 +654,8 @@ load_dir (GtkFileFolderGnomeVFS *folder_vfs)
 				  GNOME_VFS_PRIORITY_DEFAULT,
 				  directory_load_callback, folder_vfs);
   gnome_authentication_manager_pop_async ();
+
+  profile_end ("end", folder_vfs->uri);
 }
 
 static GnomeVFSFileInfo *
@@ -755,6 +762,93 @@ make_uri_canonical (const char *uri)
   return canonical;
 }
 
+/* Returns whether the MIME type of a vfs_info is "application/x-desktop" */
+static gboolean
+is_desktop_file (GnomeVFSFileInfo *vfs_info)
+{
+  return ((vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE) != 0
+	  && strcmp (gnome_vfs_file_info_get_mime_type (vfs_info), "application/x-desktop") == 0);
+}
+
+/* Returns whether a .desktop file indicates a link to a folder */
+static gboolean
+is_desktop_file_a_folder (SuckyDesktopItem *ditem)
+{
+  SuckyDesktopItemType ditem_type;
+
+  ditem_type = sucky_desktop_item_get_entry_type (ditem);
+
+  if (ditem_type != SUCKY_DESKTOP_ITEM_TYPE_LINK)
+    return FALSE;
+
+  /* FIXME: do we have to get the link URI and figure out its type?  For now,
+   * we'll just assume that it does link to a folder...
+   */
+
+  return TRUE;
+}
+
+/* Checks whether a vfs_info matches what we know about the AFS directories on
+ * the system.
+ */
+static gboolean
+is_vfs_info_an_afs_folder (GtkFileSystemGnomeVFS *system_vfs,
+			   GnomeVFSFileInfo      *vfs_info)
+{
+  return (GNOME_VFS_FILE_INFO_LOCAL (vfs_info)
+	  && (vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_DEVICE) != 0
+	  && (vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_INODE) != 0
+	  && ((system_vfs->have_afs
+	       && system_vfs->afs_statbuf.st_dev == vfs_info->device
+	       && system_vfs->afs_statbuf.st_ino == vfs_info->inode)
+	      || (system_vfs->have_net
+		  && system_vfs->net_statbuf.st_dev == vfs_info->device
+		  && system_vfs->net_statbuf.st_ino == vfs_info->inode)));
+}
+
+/* Returns the URL attribute for a .desktop file of type Link */
+static char *
+get_desktop_link_uri (const char *desktop_uri,
+		      GError    **error)
+{
+  SuckyDesktopItem *ditem;
+  const char *ditem_url;
+  char *ret_uri;
+
+  ditem = sucky_desktop_item_new_from_uri (desktop_uri, 0, error);
+  if (ditem == NULL)
+    return NULL;
+
+  ret_uri = NULL;
+
+  if (!is_desktop_file_a_folder (ditem))
+    {
+      g_set_error (error,
+		   GTK_FILE_SYSTEM_ERROR,
+		   GTK_FILE_SYSTEM_ERROR_NOT_FOLDER,
+		   _("%s is a link to something that is not a folder"),
+		   desktop_uri);
+      goto out;
+    }
+
+  ditem_url = sucky_desktop_item_get_string (ditem, SUCKY_DESKTOP_ITEM_URL);
+  if (ditem_url == NULL || strlen (ditem_url) == 0)
+    {
+      g_set_error (error,
+		   GTK_FILE_SYSTEM_ERROR,
+		   GTK_FILE_SYSTEM_ERROR_INVALID_URI,
+		   _("%s is a link without a destination location"),
+		   desktop_uri);
+      goto out;
+    }
+
+  ret_uri = g_strdup (ditem_url);
+
+ out:
+
+  sucky_desktop_item_unref (ditem);
+  return ret_uri;
+}
 
 static GtkFileFolder *
 gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
@@ -769,7 +863,8 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
   GnomeVFSMonitorHandle *monitor;
   GnomeVFSResult result;
   GnomeVFSFileInfo *vfs_info;
-  GnomeVFSFileType vfs_info_type;
+
+  profile_start ("start", (char *) path);
 
   uri = make_uri_canonical (gtk_file_path_get_string (path));
   folder_vfs = g_hash_table_lookup (system_vfs->folders, uri);
@@ -777,12 +872,14 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
     {
       folder_vfs->types |= types;
       g_free (uri);
+      profile_end ("returning cached folder", NULL);
       return g_object_ref (folder_vfs);
     }
 
   if (!gtk_file_system_get_parent (file_system, path, &parent_path, error))
     {
       g_free (uri);
+      profile_end ("returning NULL because of lack of parent", NULL);
       return NULL;
     }
 
@@ -807,6 +904,7 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
 	  if (!child)
 	    {
 	      g_free (uri);
+	      profile_end ("returning NULL because lookup_folder_child_from_uri() failed", NULL);
 	      return NULL;
 	    }
 
@@ -828,13 +926,28 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
 	  set_vfs_error (result, uri, error);
 	  gnome_vfs_file_info_unref (vfs_info);
 	  g_free (uri);
+	  profile_end ("returning NULL because couldn't get file info", NULL);
 	  return NULL;
 	}
     }
 
-  vfs_info_type = vfs_info->type;
+  if (is_desktop_file (vfs_info))
+    {
+      char *ditem_uri;
 
-  if (vfs_info_type != GNOME_VFS_FILE_TYPE_DIRECTORY)
+      ditem_uri = get_desktop_link_uri (uri, error);
+      if (ditem_uri == NULL)
+	{
+	  g_free (uri);
+	  gnome_vfs_file_info_unref (vfs_info);
+	  profile_end ("returning NULL because couldn't get desktop link uri", NULL);
+	  return NULL;
+	}
+
+      g_free (uri);
+      uri = ditem_uri;
+    }
+  else if (vfs_info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
     {
       g_set_error (error,
 		   GTK_FILE_SYSTEM_ERROR,
@@ -843,26 +956,16 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
 		   uri);
       g_free (uri);
       gnome_vfs_file_info_unref (vfs_info);
+      profile_end ("returning NULL because not a folder", NULL);
       return NULL;
     }
 
   folder_vfs = g_object_new (GTK_TYPE_FILE_FOLDER_GNOME_VFS, NULL);
 
-  /* Check for AFS */
-  if (GNOME_VFS_FILE_INFO_LOCAL (vfs_info)
-      && (vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_DEVICE) != 0
-      && (vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_INODE) != 0
-      && ((system_vfs->have_afs
-	   && system_vfs->afs_statbuf.st_dev == vfs_info->device
-	   && system_vfs->afs_statbuf.st_ino == vfs_info->inode)
-	  || (system_vfs->have_net
-	      && system_vfs->net_statbuf.st_dev == vfs_info->device
-	      && system_vfs->net_statbuf.st_ino == vfs_info->inode)))
-    folder_vfs->is_afs_or_net = TRUE;
-  else
-    folder_vfs->is_afs_or_net = FALSE;
+  folder_vfs->is_afs_or_net = is_vfs_info_an_afs_folder (system_vfs, vfs_info);
 
   gnome_vfs_file_info_unref (vfs_info);
+  vfs_info = NULL;
 
   monitor = NULL;
 
@@ -879,6 +982,7 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
 	  g_free (uri);
 	  g_object_unref (folder_vfs);
 	  set_vfs_error (result, uri, error);
+	  profile_end ("returning NULL because couldn't add monitor", NULL);
 	  return NULL;
 	}
     }
@@ -895,6 +999,8 @@ gtk_file_system_gnome_vfs_get_folder (GtkFileSystem     *file_system,
 
   g_hash_table_insert (system_vfs->folders, folder_vfs->uri, folder_vfs);
 
+  profile_end ("end", (char *) path);
+
   return GTK_FILE_FOLDER (folder_vfs);
 }
 
@@ -1094,7 +1200,7 @@ gtk_file_system_gnome_vfs_volume_get_display_name (GtkFileSystem       *file_sys
     {
       uri = gnome_vfs_volume_get_activation_uri (GNOME_VFS_VOLUME (volume));
       if (strcmp (uri, "file:///") == 0)
-	display_name = g_strdup (_("Filesystem"));
+	display_name = g_strdup (_("File System"));
       else
 	display_name = gnome_vfs_volume_get_display_name (GNOME_VFS_VOLUME (volume));
       g_free (uri);
@@ -1137,12 +1243,15 @@ icon_theme_changed (GtkIconTheme *icon_theme)
 static GdkPixbuf *
 get_cached_icon (GtkWidget   *widget,
 		 const gchar *name,
-		 gint         pixel_size)
+		 gint         pixel_size,
+		 GError     **error)
 {
   GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
   GHashTable *cache = g_object_get_data (G_OBJECT (icon_theme), "gnome-vfs-gtk-file-icon-cache");
   IconCacheElement *element;
 
+  profile_start ("start", name);
+
   if (!cache)
     {
       cache = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -1167,10 +1276,18 @@ get_cached_icon (GtkWidget   *widget,
       if (element->pixbuf)
 	g_object_unref (element->pixbuf);
       element->size = pixel_size;
-      element->pixbuf = gtk_icon_theme_load_icon (icon_theme, name,
-						  pixel_size, 0, NULL);
+
+      /* Normally, the names are raw icon names, not filenames.  If they happen
+       * to be filenames, then they likely came from a .desktop file
+       */
+      if (g_path_is_absolute (name))
+	element->pixbuf = gdk_pixbuf_new_from_file_at_size (name, pixel_size, pixel_size, error);
+      else
+	element->pixbuf = gtk_icon_theme_load_icon (icon_theme, name, pixel_size, 0, error);
     }
 
+  profile_end ("end", name);
+
   return element->pixbuf ? g_object_ref (element->pixbuf) : NULL;
 }
 
@@ -1186,6 +1303,8 @@ gtk_file_system_gnome_vfs_volume_render_icon (GtkFileSystem        *file_system,
   GdkPixbuf *pixbuf;
   GnomeVFSVolume *mounted_volume;
 
+  profile_start ("start", NULL);
+
   system_vfs = GTK_FILE_SYSTEM_GNOME_VFS (file_system);
 
   icon_name = NULL;
@@ -1218,12 +1337,14 @@ gtk_file_system_gnome_vfs_volume_render_icon (GtkFileSystem        *file_system,
 
   if (icon_name)
     {
-      pixbuf = get_cached_icon (widget, icon_name, pixel_size);
+      pixbuf = get_cached_icon (widget, icon_name, pixel_size, error);
       g_free (icon_name);
     }
   else
     pixbuf = NULL;
 
+  profile_end ("end", NULL);
+
   return pixbuf;
 }
 
@@ -1572,10 +1693,15 @@ get_vfs_info (GtkFileSystem     *file_system,
   GnomeVFSFileInfo *info;
   GtkFileSystemGnomeVFS *system_vfs;
 
+  profile_start ("start", (char *) path);
+
   system_vfs = GTK_FILE_SYSTEM_GNOME_VFS (file_system);
 
   if (!gtk_file_system_get_parent (file_system, path, &parent_path, NULL))
-    return NULL;
+    {
+      profile_end ("end with no parent", (char *) path);
+      return NULL;
+    }
   
   parent_uri = gtk_file_path_get_string (parent_path);
   info = NULL;
@@ -1608,10 +1734,42 @@ get_vfs_info (GtkFileSystem     *file_system,
   
   gtk_file_path_free (parent_path);
 
+  profile_end ("end", (char *) path);
+
   return info;
   
 }
 
+static GdkPixbuf *
+get_icon_from_desktop_file (const char   *desktop_uri,
+			    GtkWidget    *widget,
+			    gint          pixel_size,
+			    GError      **error)
+{
+  SuckyDesktopItem *ditem;
+  const char *ditem_icon_name;
+  GdkPixbuf *ret_pixbuf;
+
+  ret_pixbuf = NULL;
+
+  ditem = sucky_desktop_item_new_from_uri (desktop_uri, 0, error);
+  if (ditem == NULL)
+    goto out;
+
+  ditem_icon_name = sucky_desktop_item_get_string (ditem, SUCKY_DESKTOP_ITEM_ICON);
+  if (ditem_icon_name == NULL || strlen (ditem_icon_name) == 0)
+    goto out;
+
+  ret_pixbuf = get_cached_icon (widget, ditem_icon_name, pixel_size, error);
+
+ out:
+
+  if (ditem)
+    sucky_desktop_item_unref (ditem);
+
+  return ret_pixbuf;
+}
+
 static GdkPixbuf *
 gtk_file_system_gnome_vfs_render_icon (GtkFileSystem     *file_system,
 				       const GtkFilePath *path,
@@ -1624,14 +1782,25 @@ gtk_file_system_gnome_vfs_render_icon (GtkFileSystem     *file_system,
   const char *uri;
   GdkPixbuf *pixbuf;
   char *icon_name;
-  GnomeVFSFileInfo *info;
+  GnomeVFSFileInfo *vfs_info;
+
+  profile_start ("start", (char *) path);
 
   system_vfs = GTK_FILE_SYSTEM_GNOME_VFS (file_system);
 
   pixbuf = NULL;
 
-  info = get_vfs_info (file_system, path, GTK_FILE_INFO_MIME_TYPE);
+  vfs_info = get_vfs_info (file_system, path, GTK_FILE_INFO_MIME_TYPE);
   uri = gtk_file_path_get_string (path);
+
+  if (is_desktop_file (vfs_info))
+    {
+      pixbuf = get_icon_from_desktop_file (uri, widget, pixel_size, error);
+      gnome_vfs_file_info_unref (vfs_info);
+      profile_end ("end with icon from desktop file", (char *) path);
+      return pixbuf;
+    }
+
   if (strcmp (uri, system_vfs->desktop_uri) == 0)
     icon_name = g_strdup ("gnome-fs-desktop");
   else if (strcmp (uri, system_vfs->home_uri) == 0)
@@ -1641,20 +1810,22 @@ gtk_file_system_gnome_vfs_render_icon (GtkFileSystem     *file_system,
 				   NULL,
 				   uri,
 				   NULL,
-				   info,
-				   info->mime_type,
+				   vfs_info,
+				   vfs_info->mime_type,
 				   GNOME_ICON_LOOKUP_FLAGS_NONE,
 				   NULL);
   if (icon_name)
     {
-      pixbuf = get_cached_icon (widget, icon_name, pixel_size);
+      pixbuf = get_cached_icon (widget, icon_name, pixel_size, error);
       g_free (icon_name);
     }
   else
-    pixbuf = NULL;
+    pixbuf = NULL; /* FIXME: in this case, we are missing GError information */
 
-  if (info)
-    gnome_vfs_file_info_unref (info);
+  if (vfs_info)
+    gnome_vfs_file_info_unref (vfs_info);
+
+  profile_end ("end", (char *) path);
 
   return pixbuf;
 }
@@ -1812,11 +1983,14 @@ gtk_file_system_gnome_vfs_insert_bookmark (GtkFileSystem     *file_system,
   gboolean result;
   GError *err;
 
+  profile_start ("start", (char *) path);
+
   err = NULL;
   if (!bookmark_list_read (&bookmarks, &err) && err->code != G_FILE_ERROR_NOENT)
     {
       g_propagate_error (error, err);
       g_error_free (err);
+      profile_end ("end; could not read bookmarks list", NULL);
       return FALSE;
     }
 
@@ -1864,6 +2038,8 @@ gtk_file_system_gnome_vfs_insert_bookmark (GtkFileSystem     *file_system,
   g_free (uri);
   bookmark_list_free (bookmarks);
 
+  profile_end ("end", (char *) path);
+
   return result;
 }
 
@@ -1877,8 +2053,13 @@ gtk_file_system_gnome_vfs_remove_bookmark (GtkFileSystem     *file_system,
   GSList *l;
   gboolean result;
 
+  profile_start ("start", (char *) path);
+
   if (!bookmark_list_read (&bookmarks, error))
-    return FALSE;
+    {
+      profile_end ("end, could not read bookmarks list", NULL);
+      return FALSE;
+    }
 
   result = FALSE;
 
@@ -1925,6 +2106,8 @@ gtk_file_system_gnome_vfs_remove_bookmark (GtkFileSystem     *file_system,
   g_free (uri);
   bookmark_list_free (bookmarks);
 
+  profile_end ("end", (char *) path);
+
   return result;
 }
 
@@ -1935,8 +2118,13 @@ gtk_file_system_gnome_vfs_list_bookmarks (GtkFileSystem *file_system)
   GSList *result;
   GSList *l;
 
+  profile_start ("start", NULL);
+
   if (!bookmark_list_read (&bookmarks, NULL))
-    return NULL;
+    {
+      profile_end ("end, could not read bookmarks list", NULL);
+      return NULL;
+    }
 
   result = NULL;
 
@@ -1954,6 +2142,9 @@ gtk_file_system_gnome_vfs_list_bookmarks (GtkFileSystem *file_system)
   bookmark_list_free (bookmarks);
 
   result = g_slist_reverse (result);
+
+  profile_end ("end", NULL);
+
   return result;
 }
 
@@ -1965,9 +2156,14 @@ gtk_file_system_gnome_vfs_get_bookmark_label (GtkFileSystem     *file_system,
   gchar *label;
   GSList *l;
   gchar *bookmark, *space, *uri;
+
+  profile_start ("start", (char *) path);
   
   if (!bookmark_list_read (&bookmarks, NULL))
-    return NULL;
+    {
+      profile_end ("end, could not read bookmarks list", NULL);
+      return NULL;
+    }
 
   uri = gtk_file_system_path_to_uri (file_system, path);
 
@@ -1988,6 +2184,8 @@ gtk_file_system_gnome_vfs_get_bookmark_label (GtkFileSystem     *file_system,
   g_free (uri);
   bookmark_list_free (bookmarks);
 
+  profile_end ("end", (char *) path);
+
   return label;
 }
 
@@ -2001,8 +2199,13 @@ gtk_file_system_gnome_vfs_set_bookmark_label (GtkFileSystem     *file_system,
   gchar *bookmark, *space, *uri;
   gboolean found;
 
+  profile_start ("start", (char *) path);
+
   if (!bookmark_list_read (&bookmarks, NULL))
-    return;
+    {
+      profile_end ("end, could not read bookmarks list", NULL);
+      return;
+    }
 
   uri = gtk_file_system_path_to_uri (file_system, path);
 
@@ -2041,112 +2244,10 @@ gtk_file_system_gnome_vfs_set_bookmark_label (GtkFileSystem     *file_system,
   
   g_free (uri);
   bookmark_list_free (bookmarks);
+
+  profile_end ("end", NULL);
 }
 
-#ifdef USE_GCONF
-static gboolean
-gtk_file_system_gnome_vfs_add_bookmark (GtkFileSystem     *file_system,
-					const GtkFilePath *path,
-					GError           **error)
-{
-  GtkFileSystemGnomeVFS *system_vfs;
-  GSList *list, *l;
-  gboolean result;
-
-  system_vfs = GTK_FILE_SYSTEM_GNOME_VFS (file_system);
-
-  list = NULL;
-
-  for (l = system_vfs->bookmarks; l; l = l->next)
-    {
-      GtkFilePath *p;
-      const char *str;
-
-      p = l->data;
-      str = gtk_file_path_get_string (p);
-
-      list = g_slist_prepend (list, (gpointer) str);
-    }
-
-  list = g_slist_prepend (list, (gpointer) gtk_file_path_get_string (path));
-  list = g_slist_reverse (list);
-
-  result = gconf_client_set_list (system_vfs->client, BOOKMARKS_KEY,
-				  GCONF_VALUE_STRING,
-				  list,
-				  error);
-
-  g_slist_free (list);
-
-  return result;
-}
-
-static gboolean
-gtk_file_system_gnome_vfs_remove_bookmark (GtkFileSystem     *file_system,
-					   const GtkFilePath *path,
-					   GError           **error)
-{
-  GtkFileSystemGnomeVFS *system_vfs;
-  GSList *list, *l;
-  gboolean found;
-  gboolean result;
-
-  system_vfs = GTK_FILE_SYSTEM_GNOME_VFS (file_system);
-
-  list = NULL;
-  found = FALSE;
-
-  for (l = system_vfs->bookmarks; l; l = l->next)
-    {
-      GtkFilePath *p;
-
-      p = l->data;
-      if (gtk_file_path_compare (path, p) != 0)
-	{
-	  const char *str;
-
-	  str = gtk_file_path_get_string (p);
-	  list = g_slist_prepend (list, (gpointer) str);
-	}
-      else
-	found = TRUE;
-    }
-
-  if (found)
-    {
-      list = g_slist_reverse (list);
-
-      result = gconf_client_set_list (system_vfs->client, BOOKMARKS_KEY,
-				      GCONF_VALUE_STRING,
-				      list,
-				      error);
-    }
-  else
-    {
-      g_set_error (error,
-		   GTK_FILE_SYSTEM_ERROR,
-		   GTK_FILE_SYSTEM_ERROR_FAILED,
-		   "Path %s is not in bookmarks",
-		   gtk_file_path_get_string (path));
-      result = FALSE;
-    }
-
-  g_slist_free (list);
-
-  return result;
-}
-
-static GSList *
-gtk_file_system_gnome_vfs_list_bookmarks (GtkFileSystem *file_system)
-{
-  GtkFileSystemGnomeVFS *system_vfs;
-
-  system_vfs = GTK_FILE_SYSTEM_GNOME_VFS (file_system);
-
-  return gtk_file_paths_copy (system_vfs->bookmarks);
-}
-#endif
-
 /*
  * GtkFileFolderGnomeVFS
  */
@@ -2241,9 +2342,14 @@ lookup_folder_child_from_uri (GtkFileFolder *folder,
   GnomeVFSFileInfo *vfs_info;
   GnomeVFSResult result;
 
+  profile_start ("start", uri);
+
   child = g_hash_table_lookup (folder_vfs->children, uri);
   if (child)
-    return child;
+    {
+      profile_end ("end, returning cached child", uri);
+      return child;
+    }
 
   /* We may not have loaded the file's information yet.
    */
@@ -2257,6 +2363,7 @@ lookup_folder_child_from_uri (GtkFileFolder *folder,
     {
       set_vfs_error (result, uri, error);
       gnome_vfs_file_info_unref (vfs_info);
+      profile_end ("end, could not get file info", uri);
       return NULL;
     }
   else
@@ -2271,6 +2378,8 @@ lookup_folder_child_from_uri (GtkFileFolder *folder,
       g_signal_emit_by_name (folder_vfs, "files-added", uris);
       g_slist_free (uris);
 
+      profile_end ("end", uri);
+
       return child;
     }
 }
@@ -2298,13 +2407,15 @@ gtk_file_folder_gnome_vfs_get_info (GtkFileFolder     *folder,
   GtkFileFolderGnomeVFS *folder_vfs = GTK_FILE_FOLDER_GNOME_VFS (folder);
   const gchar *uri = gtk_file_path_get_string (path);
   FolderChild *child;
+  GtkFileInfo *file_info;
+
+  profile_start ("start", (char *) path);
 
   if (!path)
     {
       GnomeVFSURI *vfs_uri;
       GnomeVFSResult result;
       GnomeVFSFileInfo *info;
-      GtkFileInfo *file_info;
 
       vfs_uri = gnome_vfs_uri_new (folder_vfs->uri);
       g_assert (vfs_uri != NULL);
@@ -2322,18 +2433,25 @@ gtk_file_folder_gnome_vfs_get_info (GtkFileFolder     *folder,
 	  set_vfs_error (result, folder_vfs->uri, error);
 	}
       else
-	file_info = info_from_vfs_info (folder_vfs->uri, info, GTK_FILE_INFO_ALL);
+	file_info = info_from_vfs_info (folder_vfs->uri, info, GTK_FILE_INFO_ALL, error);
 
       gnome_vfs_file_info_unref (info);
+
+      profile_end ("end for non-child info", (char *) path);
+
       return file_info;
     }
 
   child = lookup_folder_child (folder, path, error);
 
   if (child)
-    return info_from_vfs_info (uri, child->info, folder_vfs->types);
+    file_info = info_from_vfs_info (uri, child->info, folder_vfs->types, error);
   else
-    return NULL;
+    file_info = NULL;
+
+  profile_end ("end", (char *) path);
+
+  return file_info;
 }
 
 static void
@@ -2354,6 +2472,8 @@ gtk_file_folder_gnome_vfs_list_children (GtkFileFolder  *folder,
 {
   GtkFileFolderGnomeVFS *folder_vfs = GTK_FILE_FOLDER_GNOME_VFS (folder);
 
+  profile_start ("start", folder_vfs->uri);
+
   if (folder_vfs->is_afs_or_net)
     load_afs_dir (folder_vfs);
   else
@@ -2363,6 +2483,8 @@ gtk_file_folder_gnome_vfs_list_children (GtkFileFolder  *folder,
 
   g_hash_table_foreach (folder_vfs->children, list_children_foreach, children);
 
+  profile_end ("end", folder_vfs->uri);
+
   return TRUE;
 }
 
@@ -2379,14 +2501,12 @@ static GnomeVFSFileInfoOptions
 get_options (GtkFileInfoType types)
 {
   GnomeVFSFileInfoOptions options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-#if 0
-  if ((types & GTK_FILE_INFO_MIME_TYPE) || (types & GTK_FILE_INFO_ICON))
-#else
-  if (types & GTK_FILE_INFO_MIME_TYPE)
-#endif
-    {
-      options |= GNOME_VFS_FILE_INFO_GET_MIME_TYPE;
-    }
+
+  /* We need the MIME type regardless of what was requested, because we need to
+   * be able to distinguish .desktop files (application/x-desktop).  FIXME: can
+   * we get away with passing GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE?
+   */
+  options |= GNOME_VFS_FILE_INFO_GET_MIME_TYPE;
 
   return options;
 }
@@ -2394,15 +2514,41 @@ get_options (GtkFileInfoType types)
 static GtkFileInfo *
 info_from_vfs_info (const gchar      *uri,
 		    GnomeVFSFileInfo *vfs_info,
-		    GtkFileInfoType   types)
+		    GtkFileInfoType   types,
+		    GError          **error)
 {
   GtkFileInfo *info = gtk_file_info_new ();
-  char *local_file;
-  char *display_name;
+  gboolean is_desktop;
+  SuckyDesktopItem *ditem;
+
+  /* Desktop files are special.  They can override the name, type, icon, etc. */
+
+  is_desktop = is_desktop_file (vfs_info);
+
+  ditem = NULL; /* shut up GCC */
+
+  if (is_desktop)
+    {
+      ditem = sucky_desktop_item_new_from_uri (uri, 0, error);
+      if (ditem == NULL)
+	return NULL;
+    }
+
+  /* Display name */
 
   if (types & GTK_FILE_INFO_DISPLAY_NAME)
     {
-      if (!vfs_info->name || strcmp (vfs_info->name, "/") == 0)
+      if (is_desktop)
+	{
+	  const char *name;
+
+	  name = sucky_desktop_item_get_localestring (ditem, SUCKY_DESKTOP_ITEM_NAME);
+	  if (name != NULL)
+	    gtk_file_info_set_display_name (info, name);
+	  else
+	    goto fetch_name_from_uri;
+	}
+      else if (!vfs_info->name || strcmp (vfs_info->name, "/") == 0)
 	{
 	  if (strcmp (uri, "file:///") == 0)
 	    gtk_file_info_set_display_name (info, "/");
@@ -2411,6 +2557,11 @@ info_from_vfs_info (const gchar      *uri,
 	}
       else
 	{
+	  char *local_file;
+	  char *display_name;
+
+	fetch_name_from_uri:
+
 	  local_file = gnome_vfs_get_local_path_from_uri (uri);
 	  if (local_file != NULL)
 	    {
@@ -2428,21 +2579,52 @@ info_from_vfs_info (const gchar      *uri,
 	}
     }
 
-  gtk_file_info_set_is_hidden (info, vfs_info->name && vfs_info->name[0] == '.');
-  gtk_file_info_set_is_folder (info, vfs_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
+  /* Hidden */
 
-#if 0
-  if ((types & GTK_FILE_INFO_MIME_TYPE) || (types & GTK_FILE_INFO_ICON))
-#else
-  if (types & GTK_FILE_INFO_MIME_TYPE)
-#endif
+  if (types & GTK_FILE_INFO_IS_HIDDEN)
     {
-      gtk_file_info_set_mime_type (info, vfs_info->mime_type);
+      gboolean is_hidden;
+
+      if (is_desktop)
+	is_hidden = sucky_desktop_item_get_boolean (ditem, SUCKY_DESKTOP_ITEM_HIDDEN);
+      else
+	is_hidden = (vfs_info->name && vfs_info->name[0] == '.');
+
+      gtk_file_info_set_is_hidden (info, is_hidden);
+    }
+
+  /* Folder */
+
+  if (types & GTK_FILE_INFO_IS_FOLDER)
+    {
+      gboolean is_folder;
+
+      if (is_desktop)
+	is_folder = is_desktop_file_a_folder (ditem);
+      else
+	is_folder = (vfs_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
+
+      gtk_file_info_set_is_folder (info, is_folder);
+    }
+
+  if (types & GTK_FILE_INFO_MIME_TYPE)
+    {
+      const char *mime_type;
+
+      if (is_desktop)
+	mime_type = "application/x-desktop"; /* FIXME: do we have to get the link URI and figure out its type? */
+      else
+	mime_type = vfs_info->mime_type;
+
+      gtk_file_info_set_mime_type (info, mime_type);
     }
 
   gtk_file_info_set_modification_time (info, vfs_info->mtime);
   gtk_file_info_set_size (info, vfs_info->size);
 
+  if (is_desktop)
+    sucky_desktop_item_unref (ditem);
+
   return info;
 }
 
@@ -2615,6 +2797,8 @@ directory_load_callback (GnomeVFSAsyncHandle *handle,
   GSList *added_uris = NULL;
   GSList *changed_uris = NULL;
 
+  profile_start ("start", NULL);
+
   /* This is called from an idle, we need to protect is as such */
   gdk_threads_enter ();
 
@@ -2623,7 +2807,11 @@ directory_load_callback (GnomeVFSAsyncHandle *handle,
    * directory_load_callback is just being called
    */
   if (folder_vfs->children == NULL)
-    return;
+    {
+      profile_end ("end", NULL);
+      gdk_threads_leave ();
+      return;
+    }
   
   for (tmp_list = list; tmp_list; tmp_list = tmp_list->next)
     {
@@ -2686,6 +2874,8 @@ directory_load_callback (GnomeVFSAsyncHandle *handle,
     }
 
   gdk_threads_leave ();
+
+  profile_end ("end", NULL);
 }
 
 static void
@@ -2698,12 +2888,18 @@ monitor_callback (GnomeVFSMonitorHandle   *handle,
   GtkFileFolderGnomeVFS *folder_vfs = user_data;
   GSList *uris;
 
+  profile_start ("start", info_uri);
+
   /* This is called from an idle, we need to protect is as such */
   gdk_threads_enter ();
 
   /* Check if destroyed */
   if (folder_vfs->children == NULL)
-    return;
+    {
+      gdk_threads_leave ();
+      profile_end ("end", info_uri);
+      return;
+    }
   
   switch (event_type)
     {
@@ -2767,6 +2963,8 @@ monitor_callback (GnomeVFSMonitorHandle   *handle,
     }
 
   gdk_threads_leave ();
+
+  profile_end ("end", info_uri);
 }
 
 static gboolean
@@ -2792,64 +2990,6 @@ has_valid_scheme (const char *uri)
   return *p == ':';
 }
 
-#ifdef USE_GCONF
-/* Sets the bookmarks list from a GConfValue */
-static void
-set_bookmarks_from_value (GtkFileSystemGnomeVFS *system_vfs,
-			  GConfValue            *value,
-			  gboolean               emit_changed)
-{
-  gtk_file_paths_free (system_vfs->bookmarks);
-  system_vfs->bookmarks = NULL;
-
-  if (value && gconf_value_get_list_type (value) == GCONF_VALUE_STRING)
-    {
-      GSList *list, *l;
-
-      list = gconf_value_get_list (value);
-
-      for (l = list; l; l = l->next)
-	{
-	  GConfValue *v;
-	  const char *uri;
-	  GtkFilePath *path;
-
-	  v = l->data;
-	  uri = gconf_value_get_string (v);
-	  path = gtk_file_system_uri_to_path (GTK_FILE_SYSTEM (system_vfs), uri);
-
-	  if (!path)
-	    continue;
-
-	  system_vfs->bookmarks = g_slist_prepend (system_vfs->bookmarks, path);
-	}
-
-      system_vfs->bookmarks = g_slist_reverse (system_vfs->bookmarks);
-    }
-
-  if (emit_changed)
-    g_signal_emit_by_name (system_vfs, "bookmarks-changed");
-}
-
-static void
-client_notify_cb (GConfClient *client,
-		  guint        cnxn_id,
-		  GConfEntry  *entry,
-		  gpointer     data)
-{
-  GtkFileSystemGnomeVFS *system_vfs;
-  GConfValue *value;
-
-  system_vfs = GTK_FILE_SYSTEM_GNOME_VFS (data);
-
-  if (strcmp (gconf_entry_get_key (entry), BOOKMARKS_KEY) != 0)
-    return;
-
-  value = gconf_entry_get_value (entry);
-  set_bookmarks_from_value (system_vfs, value, TRUE);
-}
-#endif
-
 /* GtkFileSystem module calls */
 
 void fs_module_init (GTypeModule    *module);
